﻿using System;
using System.Linq.Expressions;
using System.Reflection;
using System.Linq;

namespace EFCodeFirstExtensions
{
  public class OrderByClause<T, TProperty> : EFCodeFirstExtensions.IOrderByClause<T>
    where T : class, new()
  {
    private OrderByClause()
    {

    }

    public OrderByClause(
      Expression<Func<T, TProperty>> orderBy,
      SortDirection sortDirection = SortDirection.Ascending)
    {
      OrderBy = orderBy;
      SortDirection = sortDirection;
    }

    /// <summary>
    /// Order by expression
    /// </summary>
    private Expression<Func<T, TProperty>> OrderBy { get; set; }

    /// <summary>
    /// Sort direction
    /// </summary>
    private SortDirection SortDirection { get; set; }

    public IOrderedQueryable<T> ApplySort(IQueryable<T> query, bool firstSort)
    {
      if (SortDirection == EFCodeFirstExtensions.SortDirection.Ascending)
      {
        if (firstSort)
        {
          return query.OrderBy(OrderBy); 
        }
        else
        {
          return ((IOrderedQueryable<T>)query).ThenBy(OrderBy);
        }
      }
      else
      {
        if (firstSort)
        {
          return query.OrderByDescending(OrderBy);
        }
        else
        {
          return ((IOrderedQueryable<T>)query).ThenByDescending(OrderBy);
        }
      }

    }

  }
}
